import httpx
from ruishi.models.request import *
from ruishi.models.response import *

from typing import Tuple


class Ruishi:
    BASE_URL = 'https://acs.corelines.cn'

    class Auth(httpx.Auth):
        def __init__(self, token):
            self.token = token

        def auth_flow(self, request):
            request.headers['token'] = self.token
            yield request

    def __init__(self, username: str, password, token=''):
        self.client = httpx.Client(base_url=Ruishi.BASE_URL)
        self.async_client = httpx.AsyncClient(base_url=Ruishi.BASE_URL)
        self.username = username
        self.password = password
        if token:
            self.auth = Ruishi.Auth(token)
        else:
            self.login()

    def login(self) -> Tuple[str, LoginResponse.Content.UserDetail.User]:
        """
        :return:更新授权并返回token
        """
        url = '/auth/login'
        req = LoginRequest(account=self.username, password=self.password)
        res = self.client.post(url, json=ujson.loads(req.json()))
        res.raise_for_status()
        res = LoginResponse(**res.json())
        self.auth = Ruishi.Auth(res.content.token)
        return res.content.token, res.content.userDetail.user

    def get_room_list(self) -> List[RoomListResponse.Content.Room]:
        """
        :return:返回房间信息列表
        """
        url = '/control/community/v1/room/list'
        req = RoomListRequest()
        res = self.client.post(url, json=ujson.loads(req.json()), auth=self.auth)
        res.raise_for_status()
        res = RoomListResponse(**res.json())
        return res.content.list

    def get_device_list(self, nodeuuids: List[UUID]) -> List[DeviceListResponse.Content.Device]:
        """
        :param nodeuuids:房间uuid列表
        :return:返回设备列表
        """
        url = '/control/device/v1/p/list'
        req = DeviceListRequest(nodeUuids=nodeuuids)
        res = self.client.post(url, json=ujson.loads(req.json()), auth=self.auth)
        res.raise_for_status()
        res = DeviceListResponse(**res.json())
        return res.content.list

    def open_door(self, devicecode: str, useruuid: Optional[str] = '', transactionno: Optional[str] = '') -> bool:
        """
        :param transactionno:
        :param useruuid:
        :param devicecode: 设备uuid
        :return:开门是否成功
        """
        url = '/control/device/v1/control'
        req = DeviceControlRequest(deviceCode=devicecode, useruuid=useruuid, transactionNo=transactionno)
        res = self.client.post(url, json=ujson.loads(req.json()), auth=self.auth)
        res.raise_for_status()
        res = DeviceControlResponse(**res.json())
        return res.status == 0


if __name__ == '__main__':
    user = Ruishi('13718481995', 'qinpeng..')
    print(user.auth.token)
